#!/usr/bin/env python
# -*- coding: utf-8 -*-
import psutil
import plugins


class Plugin(plugins.BasePlugin):
    __name__ = 'process'

    def run(self, *unused):
        process = []
        for proc in psutil.process_iter():
            try:
                pinfo = proc.as_dict(attrs=[
                    'pid', 'name', 'ppid', 'exe', 'cmdline', 'username',
                    'cpu_percent', 'memory_percent', 'io_counters'
                ])
            except psutil.NoSuchProcess:
                pass
            else:
                process.append(pinfo)
        return process


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import ConfigParser
import pickle
import sys
# import os


class BasePlugin:
    '''
    Abstract class for plugins
    '''
    __name__ = ''

    def __init__(self, agent_cache=[]):
        if isinstance(agent_cache, list):
            self.agent_cache = agent_cache
        else:
            raise TypeError('Type of agent_cache have to be list')

        # if not self.__name__:
        #     self.__name__ = os.path.splitext(os.path.basename(__file__))[0]

    def run(self, config=None):
        '''
        Virtual method for running the plugin
        '''
        pass

    def execute(self):
        '''
        Execution wrapper for the plugin
        argv[1]: ini_file
        '''
        config = None
        if len(sys.argv) > 1:
            config = ConfigParser.RawConfigParser()
            config.read(sys.argv[1])
        pickle.dump(self.run(config), sys.stdout)

    def get_agent_cache(self):
        '''
        Return agent cached value for this specific plugin.
        '''
        try:
            return self.agent_cache[0]
        except Exception:
            return {}

    def set_agent_cache(self, cache):
        '''
        Set agent cache value previously passed to this plugin instance.
        To enable caching existing agent_cache list have to be passed
        to Plugin on initialization.
        Minimally it should be list().
        Agent will be able to see only changes in zero element of agent_cache, so
        do not manually override self.agent_cache, othervice cache will not be saved!

        If self.agent_cache is not a list appropriate exception will be raised.
        '''
        try:
            self.agent_cache[0] = cache
        except IndexError:
            self.agent_cache.append(cache)

    def absolute_to_per_second(self, key, val, prev_cache):
        try:
            if val >= prev_cache[key]:
                value = \
                    (val - prev_cache[key]) / \
                    (time.time() - prev_cache['ts'])
            else:  # previous cached value should not be higher than current value (service was restarted?)
                value = val / \
                    (time.time() - prev_cache['ts'])
        except KeyError:  # No cache yet, can't calculate
            value = 0
        return value


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import urllib2
import time
import plugins
import re


class Plugin(plugins.BasePlugin):
    __name__ = 'httpd'

    def run(self, config):
        '''
        Apache/httpd status page metrics
        '''

        prev_cache = {}
        next_cache = dict()
        next_cache['ts'] = time.time()
        prev_cache = self.get_agent_cache()  # Get absolute values from previous check

        try:
            data = urllib2.urlopen(config.get('httpd', 'status_page_url')).read()
        except Exception, e:
            return False

        exp = re.compile('^([A-Za-z ]+):\s+(.+)$')
        results = {}
        def parse_score_board(sb):

            ret = []

            ret.append(('IdleWorkers', sb.count('_')))
            ret.append(('ReadingWorkers', sb.count('R')))
            ret.append(('WritingWorkers', sb.count('W')))
            ret.append(('KeepaliveWorkers', sb.count('K')))
            ret.append(('DnsWorkers', sb.count('D')))
            ret.append(('ClosingWorkers', sb.count('C')))
            ret.append(('LoggingWorkers', sb.count('L')))
            ret.append(('FinishingWorkers', sb.count('G')))
            ret.append(('CleanupWorkers', sb.count('I')))

            return ret
        for line in data.split('\n'):
            if line:
                m = exp.match(line)
                if m:
                    k = m.group(1)
                    v = m.group(2)

                    # Ignore the following values
                    if k == 'IdleWorkers' or k == 'Server Built' or k == 'Server Built' \
                            or k == 'CurrentTime' or k == 'RestartTime' or k == 'ServerUptime' \
                            or k == 'CPULoad' or k == 'CPUUser' or k == 'CPUSystem' \
                            or k == 'CPUChildrenUser' or k == 'CPUChildrenSystem' \
                            or k == 'ReqPerSec':
                        continue

                    if k == 'Total Accesses':
                        results['requests_per_second'] = self.absolute_to_per_second(k, int(v), prev_cache)
                        next_cache['Total Accesses'] = int(v)

                    if k == 'Scoreboard':
                        for sb_kv in parse_score_board(v):
                            results[sb_kv[0]] = sb_kv[1]
                    else:
                        results[k] = v
        self.set_agent_cache(next_cache)
        return results

if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import plugins


class Plugin(plugins.BasePlugin):
    __name__ = 'loadavg'

    def run(self, *unused):
        return os.getloadavg()


if __name__ == '__main__':
    Plugin().execute()
